@@ -1,6 +1,6 @@
 # File::MMagic
 #
-# $Id: MMagic.pm,v 1.26.4.4 2005/08/27 15:20:31 knok Exp $
+# $Id: MMagic.pm,v 1.26.4.5 2006/10/13 16:59:20 opengl2772 Exp $
 #
 # This program is originated from file.kulp that is a production of The
 # Unix Reconstruction Projct.
@@ -339,7 +339,7 @@ BEGIN {
 	    t => "\t",
 	    f => "\f");
 
-$VERSION = "1.25";
+$VERSION = "1.27";
 $allowEightbit = 1;
 }
 
@@ -678,24 +678,26 @@ sub checktype_data {
     {
 	# in BSD's version, there's an effort to search from
 	# more specific to less, but I don't do that.
-	my ($token, %val);
+	my %val;
 	foreach my $type (keys %{$self->{SPECIALS}}) {
-	    my $token = '(' . 
-	      (join '|', sort {length($a) <=> length($b)} @{$self->{SPECIALS}->{$type}})
-		. ')';
-	    my $tdata = $data;
-	    if ($tdata =~ /$token/mg) {
-		$val{$type} = pos($tdata);
+	    my $matched_pos = undef;
+	    foreach my $token (@{$self->{SPECIALS}->{$type}}){ 
+		pos($data) = 0;
+		if ($data =~ /$token/mg) {
+		    my $tmp =  pos($data);
+		    if ((! defined $matched_pos) || ($matched_pos > $tmp)) {
+			$matched_pos = $tmp;
+		    }
+		}
 	    }
+	    $val{$type} = $matched_pos if $matched_pos;
 	}
-
 	# search latest match
 	if (%val) {
 	    my @skeys = sort { $val{$a} <=> $val{$b} } keys %val;
 	    $mtype = $skeys[0];
 	}
 	
-      ALLDONE:
 #	$mtype = 'text/plain' if (! defined $mtype);
     }
     if (! defined $mtype && check_binary($data)) {
@@ -760,7 +762,10 @@ sub magicMatch {
     # this item, then parse out its structure.  @$item is just the
     # raw string, line number, and subtests until we need the real info.
     # this saves time otherwise wasted parsing unused subtests.
-    $item = readMagicLine(@$item) if @$item == 3;
+    if (@$item == 3){
+        my $tmp = readMagicLine(@$item);
+        @$item = @$tmp;
+    }
 
     # $item could be undef if we ran into troubles while reading
     # the entry.
@@ -913,11 +918,15 @@ sub magicMatchStr {
     # this item, then parse out its structure.  @$item is just the
     # raw string, line number, and subtests until we need the real info.
     # this saves time otherwise wasted parsing unused subtests.
-    $item = readMagicLine(@$item) if @$item == 3;
+    if (@$item == 3){
+	my $tmp = readMagicLine(@$item);
 
-    # $item could be undef if we ran into troubles while reading
-    # the entry.
-    return unless defined($item);
+	# $item could be undef if we ran into troubles while reading
+	# the entry.
+	return unless defined($tmp);
+
+	@$item = @$tmp;
+    }
 
     # $fh is not be defined if -c.  that way we always return
     # false for every item which allows reading/checking the entire
@@ -1159,7 +1168,7 @@ sub readMagicLine {
     # or hex offset or an indirect offset specified in parenthesis
     # like (x[.[bsl]][+-][y]), or a relative offset specified by &.
     # offtype : 0 = absolute, 1 = indirect, 2 = relative
-    if ($line =~ s/^>*([&\(]?[a-flsx\.\+\-\d]+\)?)\s+(\S+)\s+//) {
+    if ($line =~ s/^>*([&\(]?[a-fA-Flsx\.\+\-\d]+\)?)\s+(\S+)\s+//) {
 	($offset,$type) = ($1,$2);
 
 	if ($offset =~ /^\(/) {
@@ -1313,7 +1322,10 @@ sub dumpMagic {
     my $entry;
     foreach $entry (@$magic) {
 	# delayed evaluation.
-	$entry = readMagicLine(@$entry) if @$entry == 3;
+        if (@$entry == 3){
+            my $tmp = readMagicLine(@$entry);
+            @$entry = @$tmp;
+        }
 
 	next if !defined($entry);
 
@@ -1377,6 +1389,57 @@ __DATA__
 # end local stuff
 #------------------------------------------------------------------------------
 
+#------------------------------------------------------------------------------
+# html:  file(1) magic for HTML (HyperText Markup Language) docs
+#
+# from Daniel Quinlan <quinlan@yggdrasil.com>
+#
+0	string		\<!DOCTYPE\ HTML	text/html
+0	string		\<!DOCTYPE\ html	text/html
+0	string		\<HEAD		text/html
+0	string		\<head		text/html
+0	string		\<TITLE		text/html
+0	string		\<title		text/html
+0       string          \<html          text/html
+0       string          \<HTML          text/html
+0	string		\<!--		text/html
+0	string		\<h1		text/html
+0	string		\<H1		text/html
+
+#------------------------------------------------------------------------------
+# mail.news:  file(1) magic for mail and news
+#
+# There are tests to ascmagic.c to cope with mail and news.
+0	string		Relay-Version: 	message/rfc822
+0	string		#!\ rnews	message/rfc822
+0	string		N#!\ rnews	message/rfc822
+0	string		Forward\ to 	message/rfc822
+0	string		Pipe\ to 	message/rfc822
+0	string		Return-Path:	message/rfc822
+0	string		Received:	message/rfc822
+0	string		Path:		message/news
+0	string		Xref:		message/news
+0	string		From:		message/rfc822
+0	string		Article 	message/news
+
+# Acrobat
+# (due to clamen@cs.cmu.edu)
+0	string		%PDF-		application/pdf
+
+# ZIP archiver
+0		string	PK				application/x-zip
+
+#------------------------------------------------------------------------------
+# msword: file(1) magic for MS Word files
+#
+# Contributor claims:
+# Reversed-engineered MS Word magic numbers
+#
+
+0	string		\376\067\0\043			application/msword
+#0	string		\320\317\021\340\241\261	application/msword
+0	string		\333\245-\0\0\0			application/msword
+
 #------------------------------------------------------------------------------
 # Java
 
@@ -1530,21 +1593,6 @@ __DATA__
 0	string		\<Book		application/x-frame
 0	string		\<Maker		application/x-frame
 
-#------------------------------------------------------------------------------
-# html:  file(1) magic for HTML (HyperText Markup Language) docs
-#
-# from Daniel Quinlan <quinlan@yggdrasil.com>
-#
-0	string		\<HEAD		text/html
-0	string		\<head		text/html
-0	string		\<TITLE		text/html
-0	string		\<title		text/html
-0       string          \<html          text/html
-0       string          \<HTML          text/html
-0	string		\<!--		text/html
-0	string		\<h1		text/html
-0	string		\<H1		text/html
-
 #------------------------------------------------------------------------------
 # images:  file(1) magic for image formats (see also "c-lang" for XPM bitmaps)
 #
@@ -1630,34 +1678,6 @@ __DATA__
 # Emacs 19
 0	string	;ELC\023\000\000\000	application/x-elc
 
-#------------------------------------------------------------------------------
-# mail.news:  file(1) magic for mail and news
-#
-# There are tests to ascmagic.c to cope with mail and news.
-0	string		Relay-Version: 	message/rfc822
-0	string		#!\ rnews	message/rfc822
-0	string		N#!\ rnews	message/rfc822
-0	string		Forward\ to 	message/rfc822
-0	string		Pipe\ to 	message/rfc822
-0	string		Return-Path:	message/rfc822
-0	string		Received:	message/rfc822
-0	string		Path:		message/news
-0	string		Xref:		message/news
-0	string		From:		message/rfc822
-0	string		Article 	message/news
-#------------------------------------------------------------------------------
-# msword: file(1) magic for MS Word files
-#
-# Contributor claims:
-# Reversed-engineered MS Word magic numbers
-#
-
-0	string		\376\067\0\043			application/msword
-#0	string		\320\317\021\340\241\261	application/msword
-0	string		\333\245-\0\0\0			application/msword
-
-
-
 #------------------------------------------------------------------------------
 # printer:  file(1) magic for printer-formatted files
 #
@@ -1669,10 +1689,6 @@ __DATA__
 # Jason's support for EPSF <jmaggard@timesdispatch.com>
 47 string  EPSF  image/eps
 
-# Acrobat
-# (due to clamen@cs.cmu.edu)
-0	string		%PDF-		application/pdf
-
 #------------------------------------------------------------------------------
 # sc:  file(1) magic for "sc" spreadsheet
 #
@@ -1833,9 +1849,6 @@ __DATA__
 2		string	-lh
 >6		string	-				application/x-lha
 
-# ZIP archiver
-0		string	PK				application/x-zip
-
 # POSIX tar archives
 257		string	ustar\0			application/x-tar
 257		string	ustar\040\040\0	application/x-gtar
@@ -1856,3 +1869,4 @@ __DATA__
 0	leshort		0xea60		application/x-arj
 # RAR archiver (Greg Roelofs, newt@uchicago.edu)
 0	string		Rar!		application/x-rar
+
