@@ -1,9 +1,9 @@
 /*
  * 
- * $Id: field.c,v 1.29.8.3 2001/12/21 05:36:40 knok Exp $
+ * $Id: field.c,v 1.29.8.11 2007/12/05 16:50:47 opengl2772 Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000,2001 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2007 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -24,9 +24,6 @@
  * 
  */
 
-#include <errno.h>
-#include <ctype.h>
-#include <stdio.h>
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -34,10 +31,17 @@
 #  include "support.h"
 #endif
 
+#include <ctype.h>
+#include <stdio.h>
+
+#ifdef HAVE_ERRNO_H
+#  include <errno.h>
+#endif
+
 #ifdef HAVE_STRING_H
-#include <string.h>
+#  include <string.h>
 #else
-#include <strings.h>
+#  include <strings.h>
 #endif
 
 #include "libnamazu.h"
@@ -48,6 +52,13 @@
 #include "replace.h"
 #include "idxname.h"
 
+/*
+ * Private variables
+ */
+
+static int cache_idx = 0, cache_num = 0;
+static struct field_cache fc[FIELD_CACHE_SIZE];
+
 /*
  *
  * Private functions
@@ -58,6 +69,7 @@ static void apply_field_alias ( char *field );
 static int is_field_safe_char ( int c );
 static void make_fullpathname_field ( int n );
 
+
 static void 
 apply_field_alias(char *field)
 {
@@ -114,7 +126,7 @@ nmz_isfield(const char *key)
         }
         key++;
     }
-    if (isalpha(*(key - 1)) && *key == ':' ) {
+    if (nmz_isalpha((unsigned char)*(key - 1)) && *key == ':' ) {
         return 1;
     }
     return 0;
@@ -151,18 +163,10 @@ nmz_get_field_name(const char *fieldpat)
 void 
 nmz_get_field_data(int idxid, int docid, const char *field, char *data) 
 {
-    char fname[BUFSIZE];
-    char tmpfield[BUFSIZE];
+    char fname[BUFSIZE] = "";
+    char tmpfield[BUFSIZE] = "";
     int i;
-    static int cache_idx = 0, cache_num = 0;
     FILE *fp_field, *fp_field_idx;
-    struct field_cache {
-	int idxid;
-	int docid;
-	char field[BUFSIZE];
-	char data[BUFSIZE];
-    };
-    static struct field_cache fc[FIELD_CACHE_SIZE];
 
     strcpy(data, ""); /* For safety. */
 
@@ -199,6 +203,7 @@ nmz_get_field_data(int idxid, int docid, const char *field, char *data)
     fp_field_idx = fopen(fname, "rb");
     if (fp_field_idx == NULL) {
         nmz_warn_printf("%s: %s", fname, strerror(errno));
+        fclose(fp_field);
 	return;
     }
 
@@ -218,11 +223,24 @@ nmz_get_field_data(int idxid, int docid, const char *field, char *data)
     fc[cache_idx].idxid = idxid;
     fc[cache_idx].docid = docid;
     strncpy(fc[cache_idx].field, tmpfield, BUFSIZE - 1);
+    fc[cache_idx].field[BUFSIZE - 1] = '\0';
     strncpy(fc[cache_idx].data, data, BUFSIZE - 1);
+    fc[cache_idx].data[BUFSIZE - 1] = '\0';
     cache_idx = (cache_idx + 1) % FIELD_CACHE_SIZE;
     if (cache_num < FIELD_CACHE_SIZE) {
 	cache_num++;
     }
 }
 
-
+void
+nmz_free_field_cache(void)
+{
+       int i;
+       for(i = 0; i < cache_num; i++) {
+               fc[i].idxid =0;
+               fc[i].docid =0;
+               strcpy(fc[i].field, "");
+               strcpy(fc[i].data, "");
+       }
+       cache_num = 0;
+}
